@@ -1,5 +1,4 @@
 
-
 #include <stdlib.h>
 
 #include "EXTERN.h"
@@ -16,32 +15,32 @@ Node * new(int child_count)
   if ((child_count < 1) || (child_count > MAX_LEVEL))
     croak("child_count out of bounds: must be between [1..%d]", MAX_LEVEL);
 
-  n = malloc(SIZE(child_count));
+#if USE_MALLOC
+  n = malloc((size_t) NODESIZE(child_count));
   if (n == NULL)
     croak("unable to a allocate memory");
+#else
+  Newc(1, n, NODESIZE(child_count), char, Node);
+#endif
 
   n->child_count  = child_count;
 
-  while (child_count--)
-    n->next[child_count] = &PL_sv_undef;
-
   n->key   = &PL_sv_undef;
   n->value = &PL_sv_undef;
 
   return n;
 }
 
-void DESTROY(Node * n)
+void DESTROY(Node* n)
 {
-  int index = n->child_count;
-
-  while (index--)
-    SvREFCNT_dec(n->next[index]);
-
   SvREFCNT_dec(n->key);
   SvREFCNT_dec(n->value);
 
+#if USE_MALLOC
   free(n);
+#else
+  Safefree(n);
+#endif
 }
 
 int child_count(Node * n)
@@ -108,5 +107,5 @@ SV* get_value(Node *n)
 
 int _allocated(Node* n)
 {
-  return SIZE(n->child_count);
+  return NODESIZE(n->child_count);
 }
