@@ -4,11 +4,32 @@ use 5.006;
 use strict;
 use warnings;
 
-our $VERSION = '0.051';
+our $VERSION = '0.06';
 
 require XSLoader;
 XSLoader::load('Tree::Node', $VERSION);
 
+require Exporter;
+
+our @ISA = qw( Exporter );
+
+our %EXPORT_TAGS = (
+  'p_node' => [qw(
+    p_new p_destroy p_allocated
+    p_child_count p_get_child p_get_child_or_null p_set_child
+    p_set_key p_get_key p_key_cmp p_set_value p_get_value
+  )],
+  'utility' => [qw(
+    _allocated_by_child_count MAX_LEVEL
+  )],
+);
+$EXPORT_TAGS{'all'} = [
+ @{ $EXPORT_TAGS{'p_node'} },
+ @{ $EXPORT_TAGS{'utility'} }
+];
+our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
+our @EXPORT    = ( );
+
 
 1;
 __END__
@@ -23,7 +44,9 @@ Tree::Node - Memory-efficient tree nodes in Perl
 
 Perl 5.6.0 or newer is required. Only core modules are used.
 
-A C compiler to is required to build the module.
+A C compiler to is required to build the module.  (There is no Pure-perl
+version because this package was written to overcome limitations of Perl.
+See the L</DESCRIPTION> section below.)
 
 =head1 INSTALLATION
 
@@ -62,6 +85,13 @@ You may ask "Why bother implementing an ordered structure such
 as a tree when Perl has hashes built-in?"  Since Perl is optimized
 for speed over memory usage, hashes (and lists) use a lot of memory.
 
+Using L<Devel::Size> for a reference, a list with four elements
+(corresponding to a key, value, and two child node pointers) will
+use at least 120 bytes.  A hash with four key/value pairs will
+use at least 228 bytes.  But an equivalent L<Tree::Node> object
+will use at least 68 bytes.  (However, see the L</KNOWN ISSUES>
+section below for caveats regarding memory usage.)
+
 So the purpose of this package is to provide a simple low-level Node
 class which can be used as a base class to implement various kinds
 of tree structures.  Each node has a key/value pair and a variable
@@ -70,12 +100,9 @@ number of "children" pointers.
 How nodes are organized or the algorithm used to organize them is
 for you to implement.
 
-There is no Pure-perl version because this package was written to
-overcome limitations of Perl.
-
 =for readme stop
 
-=head2 Methods
+=head2 Object Oritented Interface
 
 =over
 
@@ -128,6 +155,26 @@ routine.  To use numeric keys, for example:
     return ($self->key <=> $key);
   }
 
+B<Warning>: if you are also using the L</Procedural Interface>, then you
+should be aware that L</p_key_cmp> will not be inherited.  Instead, you
+should use something like the following:
+
+  {
+    no warnings 'redefine';
+
+    sub p_key_cmp {
+      my $ptr  = shift;
+      my $key  = shift;
+      return (p_key_key($ptr) <=> $key);
+    }
+
+    sub key_cmp {
+      my $self = shift;
+      my $key  = shift;
+      return (p_key_cmp($self->to_p_node), $key);
+    }
+  }
+
 =item set_value
 
   $node->set_value($value);
@@ -178,7 +225,11 @@ the beginning rather than end of the node list.
 
 =item MAX_LEVEL
 
-  $max = Tree::Node::MAX_LEVEL;
+  use Tree::Node ':utility';
+
+  ...
+
+  $max = MAX_LEVEL;
 
 Returns the maximum number of children. Defaults to the C constant
 C<UCHAR_MAX>, which is usually 255.
@@ -190,26 +241,119 @@ C<UCHAR_MAX>, which is usually 255.
 This is a utility routine which says how much space is allocated for a
 node.  It does not include the Perl overhead (see L</KNOWN ISSUES> below).
 
-=begin internal
-
 =item _allocated_by_child_count
 
-  $size = Tree::Node::_allocated_by_child_count( $child_count );
+  use Tree::Node ':utility';
+
+  ...
+
+  $size = _allocated_by_child_count( $child_count );
 
 This is a utility routine which returns the amount of space that would be
 allocated for a node with C<$child_count> children.
 
-=end internal
+=item to_p_node
+
+  $ptr = $node->to_p_node;
+
+This returns the pointer to the raw node data, which can be used in
+the L</Procedural Interface>.
+
+B<Warning>: do not mix and match object-oriented and procedural interface
+calls when reading child nodes!  Child node pointers are stored in an
+incompatible format.
 
 =back
 
+=head2 Procedural Inferface
+
+The experimental procedural interface was added in version 0.06.  The
+advantage of this interface is that there is much less overhead than the
+object-oriented interface (16 bytes instead of 45 bytes).  A disadvantage
+is that the node cannot be simply subclassed to change the L</p_key_cmp>
+function.
+
+To use the procedural interface, you must import the procedure names:
+
+  use Tree::Node ':p_node';
+
+Aside from working with pointers rather than blessed objects, the 
+procedures listed below are analagous to their object-oriented
+counterparts.
+
+However, you must manually call L</p_destroy> when you are done with
+the node, since Perl will not automatically destroy it when done.
+
+=over
+
+=item p_new
+
+  $ptr = p_new( $child_count );
+
+=item p_child_count
+
+  $child_count = p_child_count( $ptr );
+
+=item p_set_child
+
+  p_set_child( $mother_ptr, $index, $daughter_ptr );
+
+=item p_get_child
+
+  $daughter_ptr = p_get_child( $mother_ptr, $index );
+
+=item p_get_child_or_null
+
+  $daughter_ptr = p_get_child_or_null( $mother_ptr, $index );
+
+=item p_set_key
+
+  p_set_key( $ptr, $key );
+
+See L</to_p_node> for caveats about mixing interfaces.
+
+=item p_get_key
+
+  $key = p_get_key( $ptr );
+
+See L</to_p_node> for caveats about mixing interfaces.
+
+=item p_key_cmp
+
+  if (p_key_cmp( $ptr, $key ) < 0) { ... }
+
+See L</key_cmp> for caveats about mixing interfaces.
+
+=item p_set_value
+
+  p_set_value( $ptr, $value );
+
+=item p_get_value
+
+  $value = p_get_value( $ptr );
+
+=item p_allocated
+
+  $size = p_allocated($ptr);
+
+=item p_destroy
+
+  p_destroy($ptr);
+
+This unallocates the memory.  Perl will not call this automatically, so
+you must remember to manually destroy each pointer!
+
+=back
+
+=for readme continue
+
 =begin readme
 
 =head1 REVISION HISTORY
 
 The following changes have been made since the last release:
 
-=for readme include file="Changes" type="text" start="^0.05" stop="^0.04"
+=for readme include file="Changes" type="text" start="^0.06" stop="^0.05"
 
 See the F<Changes> file for a more detailed revision history.
 
@@ -219,12 +363,17 @@ See the F<Changes> file for a more detailed revision history.
 
 =head1 KNOWN ISSUES
 
-This module implements a Perl wrapper around a C struct, which involves
-a blessed reference to a pointer to the struct.  This overhead may make
-up for any memory savings that the C-based struct provided.
+This module implements a Perl wrapper around a C struct, which for the
+object-oriented inferface involves a blessed reference to a pointer to
+the struct.  This overhead of about 45 bytes may make up for any memory
+savings that the C-based struct provided!
 
-This module uses the C C<malloc> function, which may cause problems on
-versions of perl earlier than version 5.7.2.
+So if you what you are doing is implementing a simple key/value lookup,
+then you may be better off sticking with hashes.  If what you are doing
+requires a special structure that cannot be satisfied with hashes (even
+sorted hashes), then this module may be useful to you.
+
+Another alternative is to use the L</Procedural Interface>.
 
 =for readme stop
 
